"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .attachedcustomfieldcreate import (
    AttachedCustomFieldCreate,
    AttachedCustomFieldCreateTypedDict,
)
from .existingproductprice import ExistingProductPrice, ExistingProductPriceTypedDict
from .productpricecustomcreate import (
    ProductPriceCustomCreate,
    ProductPriceCustomCreateTypedDict,
)
from .productpricefixedcreate import (
    ProductPriceFixedCreate,
    ProductPriceFixedCreateTypedDict,
)
from .productpricefreecreate import (
    ProductPriceFreeCreate,
    ProductPriceFreeCreateTypedDict,
)
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ProductUpdateMetadataTypedDict = TypeAliasType(
    "ProductUpdateMetadataTypedDict", Union[str, int, bool]
)


ProductUpdateMetadata = TypeAliasType("ProductUpdateMetadata", Union[str, int, bool])


ProductUpdatePricesTypedDict = TypeAliasType(
    "ProductUpdatePricesTypedDict",
    Union[
        ExistingProductPriceTypedDict,
        ProductPriceFreeCreateTypedDict,
        ProductPriceFixedCreateTypedDict,
        ProductPriceCustomCreateTypedDict,
    ],
)


ProductUpdatePrices = TypeAliasType(
    "ProductUpdatePrices",
    Union[
        ExistingProductPrice,
        ProductPriceFreeCreate,
        ProductPriceFixedCreate,
        ProductPriceCustomCreate,
    ],
)


class ProductUpdateTypedDict(TypedDict):
    r"""Schema to update a product."""

    metadata: NotRequired[Nullable[Dict[str, ProductUpdateMetadataTypedDict]]]
    name: NotRequired[Nullable[str]]
    description: NotRequired[Nullable[str]]
    r"""The description of the product."""
    recurring_interval: NotRequired[Nullable[SubscriptionRecurringInterval]]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase. **Can only be set on legacy recurring products. Once set, it can't be changed.**"""
    is_archived: NotRequired[Nullable[bool]]
    r"""Whether the product is archived. If `true`, the product won't be available for purchase anymore. Existing customers will still have access to their benefits, and subscriptions will continue normally."""
    prices: NotRequired[Nullable[List[ProductUpdatePricesTypedDict]]]
    r"""List of available prices for this product. If you want to keep existing prices, include them in the list as an `ExistingProductPrice` object."""
    medias: NotRequired[Nullable[List[str]]]
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""
    attached_custom_fields: NotRequired[
        Nullable[List[AttachedCustomFieldCreateTypedDict]]
    ]


class ProductUpdate(BaseModel):
    r"""Schema to update a product."""

    metadata: OptionalNullable[Dict[str, ProductUpdateMetadata]] = UNSET

    name: OptionalNullable[str] = UNSET

    description: OptionalNullable[str] = UNSET
    r"""The description of the product."""

    recurring_interval: OptionalNullable[SubscriptionRecurringInterval] = UNSET
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase. **Can only be set on legacy recurring products. Once set, it can't be changed.**"""

    is_archived: OptionalNullable[bool] = UNSET
    r"""Whether the product is archived. If `true`, the product won't be available for purchase anymore. Existing customers will still have access to their benefits, and subscriptions will continue normally."""

    prices: OptionalNullable[List[ProductUpdatePrices]] = UNSET
    r"""List of available prices for this product. If you want to keep existing prices, include them in the list as an `ExistingProductPrice` object."""

    medias: OptionalNullable[List[str]] = UNSET
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""

    attached_custom_fields: OptionalNullable[List[AttachedCustomFieldCreate]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "name",
            "description",
            "recurring_interval",
            "is_archived",
            "prices",
            "medias",
            "attached_custom_fields",
        ]
        nullable_fields = [
            "metadata",
            "name",
            "description",
            "recurring_interval",
            "is_archived",
            "prices",
            "medias",
            "attached_custom_fields",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filteroperator import FilterOperator
from polar_sdk.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


ValueTypedDict = TypeAliasType("ValueTypedDict", Union[str, int, bool])


Value = TypeAliasType("Value", Union[str, int, bool])


class FilterClauseTypedDict(TypedDict):
    property: str
    operator: FilterOperator
    value: ValueTypedDict


class FilterClause(BaseModel):
    property: str

    operator: FilterOperator

    value: Value



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .customerstatebenefitgrant import (
    CustomerStateBenefitGrant,
    CustomerStateBenefitGrantTypedDict,
)
from .customerstatesubscription import (
    CustomerStateSubscription,
    CustomerStateSubscriptionTypedDict,
)
from .taxidformat import TaxIDFormat
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Union
from typing_extensions import TypeAliasType, TypedDict


CustomerStateMetadataTypedDict = TypeAliasType(
    "CustomerStateMetadataTypedDict", Union[str, int, bool]
)


CustomerStateMetadata = TypeAliasType("CustomerStateMetadata", Union[str, int, bool])


CustomerStateTaxIDTypedDict = TypeAliasType(
    "CustomerStateTaxIDTypedDict", Union[str, TaxIDFormat]
)


CustomerStateTaxID = TypeAliasType("CustomerStateTaxID", Union[str, TaxIDFormat])


class CustomerStateTypedDict(TypedDict):
    r"""A customer along with additional state information:

    * Active subscriptions
    * Active benefits
    """

    id: str
    r"""The ID of the customer."""
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    metadata: Dict[str, CustomerStateMetadataTypedDict]
    external_id: Nullable[str]
    r"""The ID of the customer in your system. This must be unique within the organization. Once set, it can't be updated."""
    email: str
    r"""The email address of the customer. This must be unique within the organization."""
    email_verified: bool
    r"""Whether the customer email address is verified. The address is automatically verified when the customer accesses the customer portal using their email address."""
    name: Nullable[str]
    r"""The name of the customer."""
    billing_address: Nullable[AddressTypedDict]
    tax_id: Nullable[List[Nullable[CustomerStateTaxIDTypedDict]]]
    organization_id: str
    r"""The ID of the organization owning the customer."""
    deleted_at: Nullable[datetime]
    r"""Timestamp for when the customer was soft deleted."""
    active_subscriptions: List[CustomerStateSubscriptionTypedDict]
    r"""The customer's active subscriptions."""
    granted_benefits: List[CustomerStateBenefitGrantTypedDict]
    r"""The customer's active benefit grants."""
    avatar_url: str


class CustomerState(BaseModel):
    r"""A customer along with additional state information:

    * Active subscriptions
    * Active benefits
    """

    id: str
    r"""The ID of the customer."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    metadata: Dict[str, CustomerStateMetadata]

    external_id: Nullable[str]
    r"""The ID of the customer in your system. This must be unique within the organization. Once set, it can't be updated."""

    email: str
    r"""The email address of the customer. This must be unique within the organization."""

    email_verified: bool
    r"""Whether the customer email address is verified. The address is automatically verified when the customer accesses the customer portal using their email address."""

    name: Nullable[str]
    r"""The name of the customer."""

    billing_address: Nullable[Address]

    tax_id: Nullable[List[Nullable[CustomerStateTaxID]]]

    organization_id: str
    r"""The ID of the organization owning the customer."""

    deleted_at: Nullable[datetime]
    r"""Timestamp for when the customer was soft deleted."""

    active_subscriptions: List[CustomerStateSubscription]
    r"""The customer's active subscriptions."""

    granted_benefits: List[CustomerStateBenefitGrant]
    r"""The customer's active benefit grants."""

    avatar_url: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "external_id",
            "name",
            "billing_address",
            "tax_id",
            "deleted_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutpricecreate import CheckoutPriceCreate, CheckoutPriceCreateTypedDict
from .checkoutproductcreate import CheckoutProductCreate, CheckoutProductCreateTypedDict
from .checkoutproductscreate import (
    CheckoutProductsCreate,
    CheckoutProductsCreateTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


CheckoutCreateTypedDict = TypeAliasType(
    "CheckoutCreateTypedDict",
    Union[
        CheckoutProductsCreateTypedDict,
        CheckoutProductCreateTypedDict,
        CheckoutPriceCreateTypedDict,
    ],
)


CheckoutCreate = TypeAliasType(
    "CheckoutCreate",
    Union[CheckoutProductsCreate, CheckoutProductCreate, CheckoutPriceCreate],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutconfirmstripe import CheckoutConfirmStripe, CheckoutConfirmStripeTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CheckoutsClientConfirmRequestTypedDict(TypedDict):
    client_secret: str
    r"""The checkout session client secret."""
    checkout_confirm_stripe: CheckoutConfirmStripeTypedDict


class CheckoutsClientConfirmRequest(BaseModel):
    client_secret: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The checkout session client secret."""

    checkout_confirm_stripe: Annotated[
        CheckoutConfirmStripe,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alreadyactivesubscriptionerror import AlreadyActiveSubscriptionErrorData
from .notopencheckout import NotOpenCheckoutData
from polar_sdk import utils
from typing import Union
from typing_extensions import TypeAliasType


CheckoutForbiddenErrorUnion = TypeAliasType(
    "CheckoutForbiddenErrorUnion",
    Union[AlreadyActiveSubscriptionErrorData, NotOpenCheckoutData],
)


class CheckoutForbiddenError(Exception):
    data: CheckoutForbiddenErrorUnion

    def __init__(self, data: CheckoutForbiddenErrorUnion):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, CheckoutForbiddenErrorUnion)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersortproperty import CustomerSortProperty
from .eventsortproperty import EventSortProperty
from .eventsource import EventSource
from .listresource_customer_ import ListResourceCustomer, ListResourceCustomerTypedDict
from .listresource_event_ import ListResourceEvent, ListResourceEventTypedDict
from .listresource_meter_ import ListResourceMeter, ListResourceMeterTypedDict
from .metersortproperty import MeterSortProperty
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CustomersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "CustomersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


CustomersListQueryParamOrganizationIDFilter = TypeAliasType(
    "CustomersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


MetadataQueryTypedDict = TypeAliasType(
    "MetadataQueryTypedDict", Union[str, int, bool, List[str], List[int], List[bool]]
)


MetadataQuery = TypeAliasType(
    "MetadataQuery", Union[str, int, bool, List[str], List[int], List[bool]]
)


class CustomersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[CustomersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    email: NotRequired[Nullable[str]]
    r"""Filter by exact email."""
    query: NotRequired[Nullable[str]]
    r"""Filter by name or email."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[CustomerSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class CustomersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[CustomersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    email: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by exact email."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by name or email."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[CustomerSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "email",
            "query",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = ["organization_id", "email", "query", "sorting", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomersListResponseTypedDict(TypedDict):
    result: ListResourceCustomerTypedDict


class CustomersListResponse(BaseModel):
    next: Callable[[], Optional[CustomersListResponse]]

    result: ListResourceCustomer


EventsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "EventsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventsListQueryParamOrganizationIDFilter = TypeAliasType(
    "EventsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventsListQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "EventsListQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


EventsListQueryParamCustomerIDFilter = TypeAliasType(
    "EventsListQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


ExternalCustomerIDFilterTypedDict = TypeAliasType(
    "ExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by external customer ID."""


ExternalCustomerIDFilter = TypeAliasType(
    "ExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by external customer ID."""


SourceFilterTypedDict = TypeAliasType(
    "SourceFilterTypedDict", Union[EventSource, List[EventSource]]
)
r"""Filter by event source."""


SourceFilter = TypeAliasType("SourceFilter", Union[EventSource, List[EventSource]])
r"""Filter by event source."""


class EventsListRequestTypedDict(TypedDict):
    start_timestamp: NotRequired[Nullable[datetime]]
    r"""Filter events after this timestamp."""
    end_timestamp: NotRequired[Nullable[datetime]]
    r"""Filter events before this timestamp."""
    organization_id: NotRequired[
        Nullable[EventsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    customer_id: NotRequired[Nullable[EventsListQueryParamCustomerIDFilterTypedDict]]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[Nullable[ExternalCustomerIDFilterTypedDict]]
    r"""Filter by external customer ID."""
    source: NotRequired[Nullable[SourceFilterTypedDict]]
    r"""Filter by event source."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[EventSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class EventsListRequest(BaseModel):
    start_timestamp: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter events after this timestamp."""

    end_timestamp: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter events before this timestamp."""

    organization_id: Annotated[
        OptionalNullable[EventsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    customer_id: Annotated[
        OptionalNullable[EventsListQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[ExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by external customer ID."""

    source: Annotated[
        OptionalNullable[SourceFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by event source."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[EventSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "start_timestamp",
            "end_timestamp",
            "organization_id",
            "customer_id",
            "external_customer_id",
            "source",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "start_timestamp",
            "end_timestamp",
            "organization_id",
            "customer_id",
            "external_customer_id",
            "source",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EventsListResponseTypedDict(TypedDict):
    result: ListResourceEventTypedDict


class EventsListResponse(BaseModel):
    next: Callable[[], Optional[EventsListResponse]]

    result: ListResourceEvent


MetersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "MetersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


MetersListQueryParamOrganizationIDFilter = TypeAliasType(
    "MetersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


class MetersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[MetersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    query: NotRequired[Nullable[str]]
    r"""Filter by name."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[MeterSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class MetersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[MetersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by name."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[MeterSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "query",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = ["organization_id", "query", "sorting", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class MetersListResponseTypedDict(TypedDict):
    result: ListResourceMeterTypedDict


class MetersListResponse(BaseModel):
    next: Callable[[], Optional[MetersListResponse]]

    result: ListResourceMeter



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod(str, Enum):
    CLIENT_SECRET_BASIC = "client_secret_basic"
    CLIENT_SECRET_POST = "client_secret_post"
    NONE = "none"


class OAuth2ClientConfigurationUpdateGrantTypes(str, Enum):
    AUTHORIZATION_CODE = "authorization_code"
    REFRESH_TOKEN = "refresh_token"


class OAuth2ClientConfigurationUpdateTypedDict(TypedDict):
    redirect_uris: List[str]
    client_name: str
    client_id: str
    token_endpoint_auth_method: NotRequired[
        OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod
    ]
    grant_types: NotRequired[List[OAuth2ClientConfigurationUpdateGrantTypes]]
    response_types: NotRequired[List[str]]
    scope: NotRequired[str]
    client_uri: NotRequired[Nullable[str]]
    logo_uri: NotRequired[Nullable[str]]
    tos_uri: NotRequired[Nullable[str]]
    policy_uri: NotRequired[Nullable[str]]


class OAuth2ClientConfigurationUpdate(BaseModel):
    redirect_uris: List[str]

    client_name: str

    client_id: str

    token_endpoint_auth_method: Optional[
        OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod
    ] = OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod.CLIENT_SECRET_POST

    grant_types: Optional[List[OAuth2ClientConfigurationUpdateGrantTypes]] = None

    response_types: Optional[List[str]] = None

    scope: Optional[str] = (
        "openid profile email user:read organizations:read organizations:write custom_fields:read custom_fields:write discounts:read discounts:write checkout_links:read checkout_links:write checkouts:read checkouts:write products:read products:write benefits:read benefits:write events:read events:write meters:read meters:write files:read files:write subscriptions:read subscriptions:write customers:read customers:write customer_sessions:write orders:read refunds:read refunds:write metrics:read webhooks:read webhooks:write external_organizations:read license_keys:read license_keys:write repositories:read repositories:write issues:read issues:write customer_portal:read customer_portal:write"
    )

    client_uri: OptionalNullable[str] = UNSET

    logo_uri: OptionalNullable[str] = UNSET

    tos_uri: OptionalNullable[str] = UNSET

    policy_uri: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "token_endpoint_auth_method",
            "grant_types",
            "response_types",
            "scope",
            "client_uri",
            "logo_uri",
            "tos_uri",
            "policy_uri",
        ]
        nullable_fields = ["client_uri", "logo_uri", "tos_uri", "policy_uri"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MetricsIntervalLimitTypedDict(TypedDict):
    r"""Date interval limit to get metrics for a given interval."""

    max_days: int
    r"""Maximum number of days for this interval."""


class MetricsIntervalLimit(BaseModel):
    r"""Date interval limit to get metrics for a given interval."""

    max_days: int
    r"""Maximum number of days for this interval."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventcreatecustomer import EventCreateCustomer, EventCreateCustomerTypedDict
from .eventcreateexternalcustomer import (
    EventCreateExternalCustomer,
    EventCreateExternalCustomerTypedDict,
)
from polar_sdk.types import BaseModel
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


EventsModelTypedDict = TypeAliasType(
    "EventsModelTypedDict",
    Union[EventCreateCustomerTypedDict, EventCreateExternalCustomerTypedDict],
)


EventsModel = TypeAliasType(
    "EventsModel", Union[EventCreateCustomer, EventCreateExternalCustomer]
)


class EventsIngestTypedDict(TypedDict):
    events: List[EventsModelTypedDict]
    r"""List of events to ingest."""


class EventsIngest(BaseModel):
    events: List[EventsModel]
    r"""List of events to ingest."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersubscriptioncancel import (
    CustomerSubscriptionCancel,
    CustomerSubscriptionCancelTypedDict,
)
from .customersubscriptionupdateproduct import (
    CustomerSubscriptionUpdateProduct,
    CustomerSubscriptionUpdateProductTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


CustomerSubscriptionUpdateTypedDict = TypeAliasType(
    "CustomerSubscriptionUpdateTypedDict",
    Union[
        CustomerSubscriptionUpdateProductTypedDict, CustomerSubscriptionCancelTypedDict
    ],
)


CustomerSubscriptionUpdate = TypeAliasType(
    "CustomerSubscriptionUpdate",
    Union[CustomerSubscriptionUpdateProduct, CustomerSubscriptionCancel],
)
